# Copyright (c) 2014, Silas Sewell
# All rights reserved.
#
# This file is subject to the MIT License (see the LICENSE file).

from __future__ import unicode_literals

__copyright__ = '2014, Silas Sewell'
__version__ = '0.18.1'
